package cz.cuni.dsrg.sofa.netbeans.cdl;

import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.ExtensionList;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.UniFileLoader;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

import java.io.IOException;


/** The data loader for the Component Description Language (.cdl) file.
 *  It allows the file to be recognized.
 *
 * @author hnetynka
 */


public class CDLDataLoader extends UniFileLoader 
{
    
  public class CDLFileEntry extends org.openide.loaders.FileEntry.Format 
  {

    protected java.text.Format createFormat(FileObject fileobject, String s, String s1)
    {
      java.util.Map map = new java.util.HashMap (4);
      map.put("DATE", java.text.DateFormat.getDateInstance(1, new java.util.Locale("en","US")).format(new java.util.Date()));
      map.put("TIME", java.text.DateFormat.getTimeInstance(3).format(new java.util.Date()));
      map.put("NAME", s);
      map.put("USER", System.getProperty("user.name"));
      org.openide.util.MapFormat mapformat = new org.openide.util.MapFormat(map);
      mapformat.setLeftBrace("__");
      mapformat.setRightBrace("__");
      mapformat.setExactMatch(false);
      return mapformat;
    }

 

   CDLFileEntry(MultiDataObject multidataobject, FileObject fileobject) 
   {
      super(multidataobject, fileobject);
   }
 }



  public CDLDataLoader ()
  {
    this (CDLDataObject.class);
  }

  public CDLDataLoader (Class recognizedObject) 
  {
    super (recognizedObject);
  }



   protected FileObject findPrimaryFile(FileObject fo)
   {
      FileObject primaryFile = null;

      if(fo.isData() && fo.getExt().equals("cdl"))
      {
          primaryFile = fo;
       }
          return primaryFile;
    }

  protected void initialize ()
  {
   setDisplayName (NbBundle.getMessage (CDLDataLoader.class, "LBL_loaderName"));
   ExtensionList extensions = new ExtensionList ();
   extensions.addExtension ("cdl");
   setExtensions (extensions);

   setActions (new SystemAction[] 
   {
     SystemAction.get (OpenAction.class),
     // SystemAction.get (CustomizeBeanAction.class),
     SystemAction.get (FileSystemAction.class), null,
     //SystemAction.get (CDLCompileAction.class),
     //null,

     /*
      SystemAction.get (ExecuteAction.class),
      null,
     */

     SystemAction.get (CutAction.class),
     SystemAction.get (CopyAction.class),
     SystemAction.get (PasteAction.class),
     null,
     SystemAction.get (DeleteAction.class),
     SystemAction.get (RenameAction.class),
       null,

     /*SystemAction.get (SaveAsTemplateAction.class),
        null,
        SystemAction.get (ToolsAction.class),
     */
  
     SystemAction.get (PropertiesAction.class),
    });
   }


 protected SystemAction[] defaultActions()
    {
        return new SystemAction[]
        {
            SystemAction.get(OpenAction.class),
            SystemAction.get(FileSystemAction.class),
            null,
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            SystemAction.get(PasteAction.class),
            null,
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            null,
            SystemAction.get(SaveAsTemplateAction.class),
            null,
            SystemAction.get(ToolsAction.class),
            SystemAction.get(PropertiesAction.class),
        };
    }

    protected MultiDataObject createMultiObject (FileObject primaryFile)
    throws DataObjectExistsException, IOException 
    {
        return new CDLDataObject (primaryFile, this);
    }


    protected org.openide.loaders.MultiDataObject.Entry createPrimaryEntry(MultiDataObject multidataobject, FileObject fileobject)
    {
        return new CDLFileEntry(multidataobject, fileobject);
    }



    /* Useful if creating an EditorSupport:
    static 
    {
	FileUtil.setMIMEType ("qqq", "text/x-qqq-descriptor");
    }
    */



    // Additional user-configurable properties:
    /*
    public String getMyProp () 
    {
	return (String) getProperty ("myProp");
    }

    public void setMyProp (String nue) 
    {
	putProperty ("myProp", nue, true);
    }

    public void writeExternal (ObjectOutput out) throws IOException 
    {
	super.writeExternal (out);
	out.writeUTF (getMyProp ());
    }

    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException 
    {
	super.readExternal (in);
	setMyProp (in.readUTF ());
    }
    */

}
