package cz.cuni.dsrg.sofa.netbeans.settings;

import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;


public class SofaSettings extends SystemOption
{
   // static final long serialVersionUID = ...;

    public static final String PROP_PROFILE = "Profile";
    public static final String PROP_TIRHOST = "TIRHost";
    public static final String PROP_TIRPORT = "TIRPort";
    public static final String PROP_USEPROTOCOLS = "UseProtocols";
    public static final String PROP_TESTPROTOCOLS = "TestProtocols";
    public static final String PROP_GENCODEDIR = "GenCodeDir";

    // No constructor please!

    protected void initialize () 
    {
        // If you have more complex default values which might require
        // other parts of the module to already be installed, do not
        // put them here; e.g. make the getter return them as a
        // default if getProperty returns null. (The class might be
        // initialized partway through module installation.)

        //String fs;

        super.initialize();

        setProfile ("Default");
        setTirhost ("localhost");
        setTirport ("2000");
        setUseprotocols(false);
        setTestprotocols(false);

        // if(System.getProperty("file.separator")!=null) fs= new String(System.getProperty("file.separator")); else fs="\\";
        if(System.getProperty("code.dir")!= null) setGencodedir(System.getProperty("code.dir"));
         else if(System.getProperty("user.home")!=null) setGencodedir(System.getProperty("user.home"));   // +fs+"SOFAGeneratedCode");
          else if (System.getProperty("user.dir")!=null) setGencodedir(System.getProperty("user.dir"));   // +fs+"SOFAGeneratedCode");
           else if (System.getProperty("java.io.tmpdir")!=null) setGencodedir(System.getProperty("java.io.tmpdir")); // + fs+"SOFAGeneratedCode");
            else if(System.getProperty("os.name").matches("*.[wW][iI][nN][dD][oO][wW][sS].*") ) setGencodedir(System.getProperty("c:\\"));
            else if(System.getProperty("os.name").matches("*.[oO][sS].*")) setGencodedir(System.getProperty("c:\\")); //SOFAGen"));    // DOS & OS2
            else setGencodedir(System.getProperty("/tmp"));   // UN*XES
    }


    public String displayName()
    {
        return NbBundle.getMessage(SofaSettings.class, "LBL_tirsettings");
    }

    
    public HelpCtx getHelpCtx() 
    {
        return HelpCtx.DEFAULT_HELP;
        // If you provide context help then use:
        // return new HelpCtx (SofaSettings.class);
    }

    
    /** Default instance of this system option, for the convenience of associated classes. */
    public static SofaSettings getDefault () 
    {
        return (SofaSettings) findObject (SofaSettings.class, true);
    }

  
    public String getProfile() 
    {
        return (String) getProperty (PROP_PROFILE);
    }

   
    public void setProfile(String profile) 
    {
        // Automatically fires property changes if needed etc.:
        putProperty (PROP_PROFILE, profile, true);
        // If you need to start some service, or do something else with
        // an external effect, you should not use putProperty(...): keep
        // the data as a private static member, and manually modify that
        // variable and use firePropertyChange(...). Because if putProperty(...)
        // is used, getters and setters will be skipped during project save
        // and restore, which may cause problems.
    }
    
   
   
    public String getTirhost () 
    {
        return  (String) getProperty (PROP_TIRHOST);
    }

    public void setTirhost(String host) 
    {
        putProperty (PROP_TIRHOST, host, true);
    }
    


    public String getTirport () 
    {
        return  (String) getProperty (PROP_TIRPORT);
    }

    public void setTirport(String port) 
    {
        putProperty (PROP_TIRPORT, port, true);
    }
    


    public void setUseprotocols(boolean use) 
    {
        putProperty (PROP_USEPROTOCOLS, new Boolean (use), true);
        if (!use)
            putProperty (PROP_TESTPROTOCOLS, new Boolean (use), true);
    }
    
    public void setTestprotocols(boolean test) 
    {
        putProperty (PROP_TESTPROTOCOLS, new Boolean (test), true);
        if (test)
            putProperty (PROP_USEPROTOCOLS, new Boolean (test), true);
    }
    


    public boolean isUseprotocols () 
    {
        return ((Boolean) getProperty (PROP_USEPROTOCOLS)).booleanValue();
    }

    public boolean isTestprotocols () 
    {
        return ((Boolean) getProperty (PROP_TESTPROTOCOLS)).booleanValue();
    }


    public String getGencodedir()
    {
      return (String) getProperty(PROP_GENCODEDIR);
    }

    public void setGencodedir(String s)
    {
      putProperty(PROP_GENCODEDIR, s, true);
    }



    /** Makes that they are profile options */
    public final boolean isGlobal() 
    {
        return false;
    }
}
