package cz.cuni.dsrg.sofa.netbeans.tirbrowser;

import org.openide.ErrorManager;
import org.openide.TopManager;
import org.openide.actions.PropertiesAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;

import java.rmi.RemoteException;

/** A node with some children.
 *
 * @author Petr Hnetynka
 */


public class RepositoryRootNode extends AbstractNode 
{
   RepositoryCookie repCookie;

    public RepositoryRootNode (String repName, int port, String host, SOFA.SOFAnode.Made.TIR.Repository tir)
    {
        super (new RepositoryRootChildren ());
        setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/RepositoryRootNodeIcon");
        // Whatever is most relevant to a user:
        //setDefaultAction (Properties.class);
        // Set FeatureDescriptor stuff
        //setDisplayName (NbBundle.getMessage (RepositoryRootNode.class, "LBL_RepNode"));
        //setShortDescription (NbBundle.getMessage (RepositoryRootNode.class, "HINT_RepNode"));
        // Add cookies, e.g.:
        setName (repName); // or, super.setName if needed
        /*
        getCookieSet ().add (new OpenCookie () {
		public void open () {
		    // Open something useful...
		}
	    });
        */
        // Make reorderable:
        // getCookieSet ().add (new ReorderMe ());
        getCookieSet().add(new RepositoryCookieImpl(port, host, tir, this));
        repCookie= (RepositoryCookie) getCookie(RepositoryCookieImpl.class);
        createChildren();
    }

    
      public void setWork(SOFA.SOFAnode.Made.TIR.Repository rep)
    {
        repCookie.setWork(rep);
    }


    public void unsetWork(SOFA.SOFAnode.Made.TIR.Repository rep)
    {
        repCookie.unsetWork(rep);
    }


    // Create the popup menu:
    protected SystemAction[] createActions () 
    {
        return new SystemAction[] 
        {
                   //SystemAction.get (OpenLocalExplorerAction.class),
                   //null,
                   SystemAction.get (ManageRepositoryAction.class),
                   SystemAction.get (BeginTrAction.class),
                   SystemAction.get (AbortTrAction.class),
                   SystemAction.get (CommitTrAction.class),
                   // SystemAction.get (MyFavoriteAction.class),
                   // null,                     // separator
                   /* according to what it can do:
                   SystemAction.get (ReorderAction.class),
                   null,
                   SystemAction.get (CutAction.class),
                   SystemAction.get (CopyAction.class),
                   SystemAction.get (PasteAction.class),
                   null,
                    */
                   //SystemAction.get (DeleteAction.class),
                   null,
                   /*
                   SystemAction.get (RenameAction.class),
                   null,
                   SystemAction.get (NewAction.class),
                   null,
                   SystemAction.get (ToolsAction.class),
                   null,
                    */
                   SystemAction.get (PropertiesAction.class),
               };
    }

    public HelpCtx getHelpCtx () 
    {
        return HelpCtx.DEFAULT_HELP;
        // When you have help, change to:
        // return new HelpCtx (RepositoryRootNode.class);
    }

    public RepositoryRootChildren getRepositoryRootChildren () 
    {
        return (RepositoryRootChildren) getChildren ();
    }

    // RECOMMENDED - handle cloning specially (so as not to invoke the overhead of FilterNode):
    /*
    public Node cloneNode () 
    {
	// Try to pass in similar constructor params to what you originally got:
        return new RepositoryRootNode ();
    }
    */

    // Create a property sheet:
    
    protected Sheet createSheet () 
    {
	      Sheet sheet = super.createSheet ();
	      // Make sure there is a "Properties" set:
	      Sheet.Set props = sheet.get (Sheet.PROPERTIES); // get by name, not display name
	      if (props == null) 
	      {
            props = Sheet.createPropertiesSet ();
  	        sheet.put (props);
  	      }
        try 
        {
            PropertySupport.Reflection r = new PropertySupport.Reflection(this.repCookie, Integer.TYPE, "getTIRPort", null);
            r.setName("Port");
            props.put (r);
            r = new PropertySupport.Reflection(this.repCookie, String.class, "getTIRHost", null);
            r.setName("Host");
            props.put(r);
            r = new PropertySupport.Reflection(this.repCookie, Boolean.TYPE, "isWork", null);
            r.setName("Work");
            props.put(r);
            r = new PropertySupport.Reflection(this.repCookie, String.class, "getProfileName", null);
            r.setName("Profile");
            props.put(r);
        } 
        catch (NoSuchMethodException e) 
        {
            MiscHelps.exception(e);
        }
        return sheet;
    }
    

    

    
  
  
  /*  static Class class$(String s)
  {
        try 
        {
            return Class.forName(s);
        } 
        catch(ClassNotFoundException classnotfoundexception) 
        {
            throw new NoClassDefFoundError(classnotfoundexception.getMessage());
        }
    }*/
    
    // Permit new subnodes to be created:
    /*
    public NewType[] getNewTypes () 
    {
	return new NewType[] { new NewType () 
	{
		public String getName () 
		{
                    return NbBundle.getMessage (RepositoryRootNode.class, "LBL_NewType");
		}
		// If you have help:
		// public HelpCtx getHelpCtx () {
                //     return RepositoryRootNode.class.getName () + ".newType";
		// }
		public void create () throws IOException 
		{
		    // do whatever you need, e.g.:
                    getRepositoryRootChildren ().addKey (someNewKey);
		    // Throw an IOException if you are creating an underlying
		    // object and this fails.
		}
	    }
	};
    }
    */

    // Permit things to be pasted into this node:
    /*
    protected void createPasteTypes (final Transferable t, List l) 
    {
	// Make sure to pick up super impl, which adds intelligent node paste type:
	super.createPasteTypes (t, l);
	if (t.isDataFlavorSupported (DataFlavor.stringFlavor)) 
	{
	    l.add (new PasteType () 
	    {
		    public String getName () 
		    {
                        return NbBundle.getMessage (RepositoryRootNode.class, "LBL_PasteType");
		    }
		    // If you have help:
		    // public HelpCtx getHelpCtx () 
		    {
                    //     return RepositoryRootNode.class.getName () + ".pasteType";
		    // }
		    public Transferable paste () throws IOException 
		    {
			try 
			{
			    String data = (String) t.getTransferData (DataFlavor.stringFlavor);
			    // Or, you can look for nodes and related things in the transferable, using e.g.:
			    // Node n = NodeTransfer.node (t, NodeTransfer.COPY);
			    // Node[] ns = NodeTransfer.nodes (t, NodeTransfer.MOVE);
			    // MyCookie cookie = (MyCookie) NodeTransfer.cookie (t, NodeTransfer.COPY, MyCookie.class);
			    // do something, e.g.:
                            getRepositoryRootChildren ().addKey (data);
			    // Throw an IOException if you are creating an underlying
			    // object and this fails.
			    // To leave the clipboard as is:
			    return null;
			    // To clear the clipboard:
			    // return ExTransferable.EMPTY;
			} 
			catch (UnsupportedFlavorException ufe) 
			{
			    // Should not happen, since t said it supported this flavor, but:
			    throw new IOException (ufe.getMessage ());
			}
		    }
		});
	}
    }
    */

    
    
    
    // Handle renaming:
    /*
    public boolean canRename () 
    {
	return true;
    }


    public void setName (String nue) 
    {
	// Update visible name, fire property changes:
	super.setName (nue);
	// perform additional actions, i.e. rename underlying object
    }
    */

    // Handle deleting:
    public boolean canDestroy () 
    {
	return true;
    }
    

    public void destroy () throws java.io.IOException 
    {
	  // Actually remove the node itself and fire property changes:
	  super.destroy ();
	 // perform additional actions, i.e. delete underlying object
	 // (and don't forget about objects represented by your children!
      repCookie.destroy();
      RepositoryRootChildren ch = getRepositoryRootChildren();
      ch.remove(ch.getNodes());
    }
    



    // Handle copying and cutting specially:
    /*
    public boolean canCopy () 
    {
	return true;
    }
    
    
    public boolean canCut () 
    {
	return true;
    }


    public Transferable clipboardCopy () 
    {
	// Add to, do not replace, the default node copy flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCopy ());
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                    return RepositoryRootNode.this.getDisplayName ();
		}
	    });
	return et;
    }
    public Transferable clipboardCut () 
    {
	// Add to, do not replace, the default node cut flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCut ());
	// This is not so useful because this node will not be destroyed afterwards
	// (it is up to the paste type to decide whether to remove the "original",
	// and it is not safe to assume that getData will only be called once):
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                    return RepositoryRootNode.this.getDisplayName ();
		}
	    });
	return et;
    }
    */

    // Permit user to customize whole node at once (instead of per-property):
    /*
    public boolean hasCustomizer () 
    {
	return true;
    }


    public Component getCustomizer () 
    {
	return new MyCustomizingPanel (this);
    }
    */

    // Permit node to be reordered (you may also want to put
    // MoveUpAction and MoveDownAction on the subnodes, if you can,
    // but ReorderAction on the parent is enough):
    /*


    private class ReorderMe extends Index.Support 
    {
      
	public Node[] getNodes () 
	{
            return RepositoryRootNode.this.getChildren ().getNodes ();
	}
      
	public int getNodesCount () 
	{
	    return getNodes ().length;
	}
      
	// This assumes that there is exactly one child node per key.
	// If you are using e.g. Children.Array, you can use shortcut implementations
	// of the Index cookie.
	public void reorder (int[] perm) {
	    // Remember: {2, 0, 1} cycles three items forwards.
            List old = RepositoryRootNode.this.getRepositoryRootChildren ().myKeys;
	    if (list.size () != perm.length) throw new IllegalArgumentException ();
	    List nue = new ArrayList (perm.length);
	    for (int i = 0; i < perm.length; i++)
		nue.set (i, old.get (perm[i]));
            RepositoryRootNode.this.getRepositoryRootChildren ().setKeys (nue);
	}
      
    }
    */
    
    protected void createChildren()
    {
        TopManager tp = TopManager.getDefault();
        SOFA.SOFAnode.Made.TIR.Repository t;
        t= repCookie.getTIR();
        try
        {
            RepositoryRootChildren children = getRepositoryRootChildren();
            SOFA.SOFAnode.Made.TIR.Contained[] cnt = t.contents(null);
            for (int i=0;i<cnt.length;i++) 
            {
                SOFA.SOFAnode.Made.TIR.Identification id = cnt[i].get_identification();
                String name = MiscHelps.printKind(cnt[i].get_def_kind());
                /*
                name = name + " \"" + id.name();
                String ver = id.version();
                if (ver!=null && ver.length()!=0)
                    name = name +"?" ver;
                name = name + "\"";
                */
                children.addChild (new TIRContainerNode((SOFA.SOFAnode.Made.TIR.Container) cnt[i], name));
            }
            children.addChild (new TIRProfilesNode(t.getProfiles()));
        } 
        catch (RemoteException e) 
        {
            ErrorManager.getDefault().notify(e);
        }
        catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e)
        {
            ErrorManager.getDefault().notify(e);
        }
    }


/*    class PortProp extends Node.Property
      {
        int port;
        public PortProp (int p) 
        {
            super(Integer.TYPE);
            port = p;
            setName("Port");
        }

        public boolean canRead() { return true; }
        public boolean canWrite() { return false; }
        public Object getValue() { return port; }
        public void setValue(Object o) {port = (int) o;}
    }*/

}
