package cz.cuni.dsrg.sofa.netbeans.tirbrowser;

import org.openide.nodes.Children;
import org.openide.nodes.Node;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/** List of children of a containing node.
 * Remember to document what your permitted keys are!
 *
 * @author Petr Hnetynka
 */

public class TIRProfilesChildren extends Children.Keys 
{
    /** Optional holder for the keys, to be used when changing them dynamically. */
    protected List myKeys;

    public TIRProfilesChildren () 
    {
        myKeys = null;
    }

    protected void addNotify () 
    {
        if (myKeys != null) return;
        myKeys = new LinkedList ();
        // add whatever keys you need
        setKeys (myKeys);
    }

    protected void removeNotify () 
    {
        myKeys = null;
        setKeys (Collections.EMPTY_SET);
    }

    protected Node[] createNodes (Object key) 
    {
        TIRProfilesKey k = (TIRProfilesKey) key;
        return new Node[] { new ProfileNode(k.profile, k.name)};
    }

    /** Optional accessor method for the keys, for use by the container node or maybe subclasses. */
    protected void addKey (Object newKey) 
    {
	// Make sure some keys already exist:
	addNotify ();
	myKeys.add (newKey);
	// Ensure that the node(s) is displayed:
	refreshKey (newKey);
    }
    
    /** Optional accessor method for keys, for use by the container node or maybe subclasses. */
    protected void setMyKeys (Collection keys) 
    {
	myKeys = new LinkedList ();
	myKeys.addAll (keys);
	super.setKeys (keys);
    }
    

    // Could also write e.g. removeKey to be used by the nodes in this children.
    // Or, could listen to changes in their status (NodeAdapter.nodeDestroyed)
    // and automatically remove them from the keys list here. Etc.

}
