package cz.cuni.dsrg.sofa.netbeans.tirbrowser;

import SOFA.SOFAnode.Made.TIR.Repository;
import cz.cuni.dsrg.sofa.netbeans.settings.SofaSettings;
import org.openide.DialogDescriptor;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.actions.NewAction;
import org.openide.actions.PropertiesAction;
import org.openide.nodes.AbstractNode;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.NewType;

import java.io.IOException;
import java.rmi.Naming;

/** A node with some children.
 *
 * @author Petr Hnetynka
 */

public class TIRRootNode extends AbstractNode 
{
    OpenedWorkRepository owr;
    
    public TIRRootNode() 
    {
        super (new TIRRootNodeChildren ());
      //System.setSecurityManager(new java.rmi.RMISecurityManager());
        setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/TIRRootNodeIcon");
      // Whatever is most relevant to a user:
        setDefaultAction(SystemAction.get (PropertiesAction.class));
     // Set FeatureDescriptor stuff:
        setName ("SOFA TIR"); // or, super.setName if needed
        setDisplayName (NbBundle.getMessage (TIRRootNode.class, "LBL_node"));
        setShortDescription (NbBundle.getMessage (TIRRootNode.class, "HINT_node"));

        // Add cookies, e.g.:
        /*
        getCookieSet ().add (new OpenCookie () {
		public void open () {
		    // Open something useful...
		}
	    });
        */
        // Make reorderable:
        // getCookieSet ().add (new ReorderMe ());

        addNodeListener(new TIRRootNodeListener());
        owr = (OpenedWorkRepository) org.openide.util.SharedClassObject.findObject(OpenedWorkRepository.class, true);
    }

    
    
    // Create the popup menu:
    protected SystemAction[] createActions () 
    {
        return new SystemAction[] 
        {
                   //SystemAction.get (OpenLocalExplorerAction.class),
                   //null,
                   // SystemAction.get (MyFavoriteAction.class),
                   // null,                     // separator
                   /* according to what it can do:
                   SystemAction.get (ReorderAction.class),
                   null,
                   SystemAction.get (CutAction.class),
                   SystemAction.get (CopyAction.class),
                   SystemAction.get (PasteAction.class),
                   null,
                   SystemAction.get (DeleteAction.class),
                   SystemAction.get (RenameAction.class),
                   null,*/
                   SystemAction.get (NewAction.class),
                   null,
                   
                   //SystemAction.get (ToolsAction.class),
                   //null,
                   SystemAction.get (PropertiesAction.class),
        };
    }



    public HelpCtx getHelpCtx () 
    {
        return HelpCtx.DEFAULT_HELP;
        // When you have help, change to:
        // return new HelpCtx (TIRRootNodeNode.class);
    }

    
    
    protected TIRRootNodeChildren getTIRRootNodeChildren () 
    {
        return (TIRRootNodeChildren) getChildren ();
    }

    
    // RECOMMENDED - handle cloning specially (so as not to invoke the overhead of FilterNode):
    /*
    public Node cloneNode () 
    {
	// Try to pass in similar constructor params to what you originally got:
        return new TIRRootNodeNode ();
    }
    */

    // Create a property sheet:
    /*
    protected Sheet createSheet () 
    {
	Sheet sheet = super.createSheet ();
	// Make sure there is a "Properties" set:
	Sheet.Set props = sheet.get (Sheet.PROPERTIES); // get by name, not display name
	if (props == null) 
	{
	    props = Sheet.createPropertiesSet ();
	    sheet.put (props);
	}
	props.put (new MyProp (someParams));
    }
    */

    // Permit new subnodes to be created:
    
    public NewType[] getNewTypes () 
    {
	return new NewType[] { new NewType () 
	{
		public String getName () 
		{
                    //return NbBundle.getMessage (TIRRootNode.class, "LBL_NewRepositoryConnection");
                    return "Connection";
		}
         	
                // If you have help:
		// public HelpCtx getHelpCtx () 
	       //	{
                //     return TIRRootNodeNode.class.getName () + ".newType";
		// }
		public void create () throws IOException 
		{
		    TopManager tp = TopManager.getDefault();
                   AddNewRepositoryConnection panel = new AddNewRepositoryConnection();
                   SofaSettings settings = (SofaSettings) SystemOption.findObject(SofaSettings.class, true);

                   if(settings!=null)
                   {
                   String prop = settings.getTirhost();
                   if (prop != null)
                       panel.setHost(prop);
                     prop = settings.getTirport();
                    if (prop != null)
                        panel.setPort(prop);
                    DialogDescriptor dd = new DialogDescriptor(panel, getName());
                    java.awt.Dialog d = tp.createDialog(dd);
                    d.pack();
                    d.show();

                   if (dd.getValue() == DialogDescriptor.OK_OPTION)
                    {
                        TIRRootNodeChildren children = getTIRRootNodeChildren ();
                        java.net.InetAddress addr = null;
                        try
                        {
                          addr = java.net.InetAddress.getByName(panel.getHost());
                        }
                         catch (java.net.UnknownHostException e)
                        {
                            MiscHelps.exception(e);
                            return;
                         }
                        String nodeName = addr.getHostName() + ":" + panel.getPort();
                        if (!children.testChildKey(nodeName)) 
                        {
                            // key is not OK
                            tp.notify(new NotifyDescriptor.Message("This TIR is already connected"));
                            return;
                        }
                        Repository tir;
                        try 
                        {
                            tir = (SOFA.SOFAnode.Made.TIR.Repository) Naming.lookup("//"+nodeName+"/Repository");
                            

                              /*java.lang.Object r =  Naming.lookup("//"+nodeName+"/Repository");
                            MiscHelps.message(r.toString());
                            ClassLoader a = this.getClass().getClassLoader();
                            ClassLoader b = r.getClass().getClassLoader();
                            
                            Class[] allcls = r.getClass().getInterfaces();
                            //MiscHelps.message(Integer.toString(allcls.length));
                            for (int i=0;i<allcls.length;i++) 
                            {
                                if (allcls[i].getName().compareTo("SOFA.SOFAnode.Made.TIR.Repository")==0)
                                {
                                    b = allcls[i].getClassLoader();
                                    MiscHelps.message(allcls[i].toString() +"\n"+b.toString());
                                    if ( a == b )
                                        MiscHelps.message("Ano");
                                    else
                                        MiscHelps.message("Ne");
                                    break;
                                }
                            }
                            
                            
                            MiscHelps.message(a.toString()+"\n"+b.toString()+"\n"+ClassLoader.getSystemClassLoader());
                            
                            
                            Class x = b.loadClass("SOFA.SOFAnode.Made.TIR.Repository");
                            MiscHelps.message(x.toString());
                            TIR = (Repository) r;
                            return;*/
                        }
                          catch (Exception e)
                        {
                            MiscHelps.exception(e);
                            return;
                        }
                        children.addChild (new RepositoryRootNode(nodeName, Integer.parseInt(panel.getPort()), panel.getHost(), tir));
                    }
                   }

		    // Throw an IOException if you are creating an underlying
		    // object and this fails.
		}
       }
	};
    }


    // Permit things to be pasted into this node:
    /*
    protected void createPasteTypes (final Transferable t, List l)
    {
	// Make sure to pick up super impl, which adds intelligent node paste type:
	super.createPasteTypes (t, l);
	if (t.isDataFlavorSupported (DataFlavor.stringFlavor))
	{
	    l.add (new PasteType () {
		    public String getName ()
		    {
                        return NbBundle.getMessage (TIRRootNodeNode.class, "LBL_PasteType");
		    }
		    // If you have help:
		    // public HelpCtx getHelpCtx ()
		    {
                    //     return TIRRootNodeNode.class.getName () + ".pasteType";
		    // }
		    public Transferable paste () throws IOException
		    {
			try
			{
			    String data = (String) t.getTransferData (DataFlavor.stringFlavor);
			    // Or, you can look for nodes and related things in the transferable, using e.g.:
			    // Node n = NodeTransfer.node (t, NodeTransfer.COPY);
			    // Node[] ns = NodeTransfer.nodes (t, NodeTransfer.MOVE);
			    // MyCookie cookie = (MyCookie) NodeTransfer.cookie (t, NodeTransfer.COPY, MyCookie.class);
			    // do something, e.g.:
                            getTIRRootNodeChildren ().addKey (data);
			    // Throw an IOException if you are creating an underlying
			    // object and this fails.
			    // To leave the clipboard as is:
			    return null;
			    // To clear the clipboard:
			    // return ExTransferable.EMPTY;
			}
			catch (UnsupportedFlavorException ufe)
			{
			    // Should not happen, since t said it supported this flavor, but:
			    throw new IOException (ufe.getMessage ());
			}
		    }
		});
	}
    }
    */

    // Handle renaming:
    /*
    public boolean canRename () 
    {
	return true;
    }

    public void setName (String nue) 
    {
	// Update visible name, fire property changes:
	super.setName (nue);
	// perform additional actions, i.e. rename underlying object
    }
    */

    // Handle deleting:
    /*
    public boolean canDestroy () 
    {
	return true;
    }


    public void destroy () throws IOException 
    {
	// Actually remove the node itself and fire property changes:
	super.destroy ();
	// perform additional actions, i.e. delete underlying object
	// (and don't forget about objects represented by your children!)
    }
    */

    // Handle copying and cutting specially:
    /*
    public boolean canCopy () 
    {
	return true;
    }


    public boolean canCut () 
    {
	return true;
    }

    public Transferable clipboardCopy () 
    {
	// Add to, do not replace, the default node copy flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCopy ());
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                    return TIRRootNodeNode.this.getDisplayName ();
		}
	 });
	return et;
    }
    public Transferable clipboardCut () 
    {
	// Add to, do not replace, the default node cut flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCut ());
	// This is not so useful because this node will not be destroyed afterwards
	// (it is up to the paste type to decide whether to remove the "original",
	// and it is not safe to assume that getData will only be called once):
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) {
		protected Object getData () {
                    return TIRRootNodeNode.this.getDisplayName ();
		}
	    });
	return et;
    }
    */

    // Permit user to customize whole node at once (instead of per-property):
    /*
    public boolean hasCustomizer () 
    {
	return true;
    }
    
    public Component getCustomizer () 
    {
	return new MyCustomizingPanel (this);
    }
    */

    // Permit node to be reordered (you may also want to put
    // MoveUpAction and MoveDownAction on the subnodes, if you can,
    // but ReorderAction on the parent is enough):
    /*
    private class ReorderMe extends Index.Support 
    {
      
	public Node[] getNodes () 
	{
            return TIRRootNodeNode.this.getChildren ().getNodes ();
	}
      
	public int getNodesCount () 
	{
	    return getNodes ().length;
	}
      
	// This assumes that there is exactly one child node per key.
	// If you are using e.g. Children.Array, you can use shortcut implementations
	// of the Index cookie.
	public void reorder (int[] perm) 
	{
	    // Remember: {2, 0, 1} cycles three items forwards.
            List old = TIRRootNodeNode.this.getTIRRootNodeChildren ().myKeys;
	    if (list.size () != perm.length) throw new IllegalArgumentException ();
	    List nue = new ArrayList (perm.length);
	    for (int i = 0; i < perm.length; i++)
		nue.set (i, old.get (perm[i]));
            TIRRootNodeNode.this.getTIRRootNodeChildren ().setKeys (nue);
	}
      
    }
    */

}