package cz.cuni.dsrg.sofa.netbeans.visualdesigner.designer;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * This class is a JPanel implementing the binding layer, a layer that
 * takes care of displaying the bindings.
 *
 * @author Michael Cifka
 */
public class BindingLayer extends JPanel
{

    public BindingLayer()
    {
        bindings = new ArrayList();
        setOpaque(false);
    }

    protected void paintComponent(Graphics g)
    {
        g.setColor(Color.black);
        for(int i = 0; i < bindings.size(); i++)
        {
            Binding b = (Binding) bindings.get(i);
            drawBinding(g, b.source, b.target);
        }
    }

    public static void drawBinding(Graphics g, GrInterface source, GrInterface target)
    {
        Point[] path = BindingPath.path(
                source.getCenterLocation(), target.getCenterLocation(),
                source.getDirection(), target.getDirection(), 5, 10);
        for(int j = 0; j < path.length - 1; j++)
            g.drawLine(path[j].x, path[j].y, path[j + 1].x, path[j + 1].y);
        drawArrow(g, path[path.length - 2], path[path.length - 1]);
    }

    private static void drawArrow(Graphics g, Point ptA, Point ptB)
    {
        int dir = 0;
        if(ptA.x < ptB.x)
            dir = SwingConstants.EAST;
        else if(ptA.x > ptB.x)
            dir = SwingConstants.WEST;
        else if(ptA.y < ptB.y)
            dir = SwingConstants.SOUTH;
        else if(ptA.y > ptB.y) dir = SwingConstants.NORTH;

        int x1 = ptB.x, y1 = ptB.y, x2 = ptB.x, y2 = ptB.y;
        for(int i = 0; i < 4; i++)
        {
            if(dir == SwingConstants.EAST)
            {
                x1--;
                y1--;
                x2--;
                y2++;
            }
            else if(dir == SwingConstants.WEST)
            {
                x1++;
                y1--;
                x2++;
                y2++;
            }
            else if(dir == SwingConstants.SOUTH)
            {
                x1--;
                y1--;
                x2++;
                y2--;
            }
            else if(dir == SwingConstants.NORTH)
            {
                x1--;
                y1++;
                x2++;
                y2++;
            }
            g.drawLine(x1, y1, x2, y2);
        }
    }

    public void addBinding(GrInterface source, GrInterface target)
    {
        Binding b = new Binding(source, target);
        bindings.add(b);
    }

    public void removeBinding(GrInterface source)
    {
        for(int i = 0; i < bindings.size(); i++)
            if(((Binding) bindings.get(i)).source == source)
                bindings.remove(i);
    }

    public void removeAll()
    {
        bindings.clear();
    }

    private class Binding
    {
        Binding(GrInterface source, GrInterface target)
        {
            this.source = source;
            this.target = target;
        }

        GrInterface source;
        GrInterface target;
    }

    private ArrayList bindings;
}
