package cz.cuni.dsrg.sofa.netbeans.visualdesigner.designer;

import javax.swing.*;
import java.awt.*;

/**
 * This class computes the path to bind two interfaces. It returns an
 * array of points which represent the binding segments,
 * when interconnected.
 *
 * @author Michael Cifka
 */
public class BindingPath
{

    private static final int NORTH = SwingConstants.NORTH;
    private static final int EAST = SwingConstants.EAST;
    private static final int SOUTH = SwingConstants.SOUTH;
    private static final int WEST = SwingConstants.WEST;

    public static Point[] path(Point ptA, Point ptB,
                               int dirA, int dirB, int space, int leading)
    {
        Point ptA1 = translate(ptA, opositeDir(dirA), 1);
        Point ptB1 = translate(ptB, opositeDir(dirB), 1);
        if(distance(ptA1, ptB) < distance(ptA, ptB)) dirA = opositeDir(dirA);
        if(distance(ptA, ptB1) < distance(ptA, ptB)) dirB = opositeDir(dirB);

        Point ptC = translate(ptA, dirA, space);
        Point ptD = translate(ptB, dirB, space);
        Point ptE = translate(ptC, dirA, leading);
        Point ptF = translate(ptD, dirB, leading);

        return (Point[]) join(join(
                new Point[]{ptC}, simple(ptE, ptF, dirA, dirB)), new Point[]{ptD});
    }

    protected static Point[] simple(Point ptA, Point ptB, int dirA, int dirB)
    {
        if(dirA == NORTH || dirA == SOUTH)
        {
            if(ptA.x == ptB.x)
                return new Point[]{ptA, ptB};
            else if(dirA == opositeDir(dirB))
            {
                if(vertDir(ptA, ptB) == dirA)
                {
                    Point ptC = new Point(ptA.x, (ptA.y + ptB.y) / 2);
                    return (Point[]) (join(
                            new Point[]{ptA},
                            simple(ptC, ptB, horizDir(ptC, ptB), dirB)));
                }
                else
                    return simple(ptA, ptB, horizDir(ptA, ptB), horizDir(ptB, ptA));
            }
            else
            {
                if(ptA.y == ptB.y)
                {
                    return new Point[]{ptA, ptB};
                }
                else if(dirA == vertDir(ptA, ptB) && dirB == horizDir(ptB, ptA)
                        || dirA == dirB && dirA == vertDir(ptA, ptB))
                {
                    Point ptC = new Point(ptA.x, ptB.y);
                    return new Point[]{ptA, ptC, ptB};
                }
                else
                {
                    Point ptC = new Point(ptB.x, ptA.y);
                    return new Point[]{ptA, ptC, ptB};
                }
            }
        }
        else
        {
            if(ptA.y == ptB.y)
                return new Point[]{ptA, ptB};
            else if(dirA == opositeDir(dirB))
            {
                if(horizDir(ptA, ptB) == dirA)
                {
                    Point ptC = new Point((ptA.x + ptB.x) / 2, ptA.y);
                    return (Point[]) (join(
                            new Point[]{ptA},
                            simple(ptC, ptB, vertDir(ptC, ptB), dirB)));
                }
                else
                    return simple(ptA, ptB, vertDir(ptA, ptB), vertDir(ptB, ptA));
            }
            else
            {
                if(ptA.x == ptB.x)
                {
                    return new Point[]{ptA, ptB};
                }
                else if(dirA == horizDir(ptA, ptB) && dirB == vertDir(ptB, ptA)
                        || dirA == dirB && dirA == horizDir(ptA, ptB))
                {
                    Point ptC = new Point(ptB.x, ptA.y);
                    return new Point[]{ptA, ptC, ptB};
                }
                else
                {
                    Point ptC = new Point(ptA.x, ptB.y);
                    return new Point[]{ptA, ptC, ptB};
                }
            }
        }
    }

    // translate the point by the distance in the specified direction
    protected static Point translate(Point point, int direction, int distance)
    {
        int x = 0, y = 0;
        if(direction == NORTH)
            y = -distance;
        else if(direction == SOUTH)
            y = distance;
        if(direction == EAST)
            x = distance;
        else if(direction == WEST)
            x = -distance;
        return new Point(point.x + x, point.y + y);
    }

    // either 0 or direction for external segment necessary to bypass the point A
    protected static int external(Point ptA, Point ptB, int dirA, int dirB, int distance)
    {
        if(horizDir(ptA, ptB) == opositeDir(dirA))
        {
            int vertDist = Math.abs(ptA.y - ptB.y);
            if((dirB == NORTH || dirB == SOUTH) && vertDist < distance)
                return dirB;
            else if(dirB == dirA && vertDist < distance)
                return vertDir(ptA, ptB) != 0 ? opositeDir(vertDir(ptA, ptB)) : NORTH;
            else if(dirB == opositeDir(dirA) && vertDist < 2 * distance)
                return vertDir(ptA, ptB) != 0 ? opositeDir(vertDir(ptA, ptB)) : NORTH;
        }
        else if(vertDir(ptA, ptB) == opositeDir(dirA))
        {
            int horizDist = Math.abs(ptA.x - ptB.x);
            if((dirB == EAST || dirB == WEST) && horizDist < distance)
                return dirB;
            else if(dirB == dirA && horizDist < distance)
                return horizDir(ptA, ptB) != 0 ? opositeDir(horizDir(ptA, ptB)) : EAST;
            else if(dirB == opositeDir(dirA) && horizDist < 2 * distance)
                return horizDir(ptA, ptB) != 0 ? opositeDir(horizDir(ptA, ptB)) : EAST;
        }
        return 0;
    }

    // horizonatal direction from A to B; either EAST, WEST or 0
    protected static int horizDir(Point ptA, Point ptB)
    {
        if(ptA.x > ptB.x)
            return WEST;
        else if(ptA.x < ptB.x)
            return EAST;
        else
            return 0;
    }

    // vertical direction from A to B; either NORTH, SOUTH or 0
    protected static int vertDir(Point ptA, Point ptB)
    {
        if(ptA.y > ptB.y)
            return NORTH;
        else if(ptA.y < ptB.y)
            return SOUTH;
        else
            return 0;
    }

    // oposite direction to dir
    protected static int opositeDir(int dir)
    {
        switch(dir)
        {
            case NORTH:
                return SOUTH;
            case EAST:
                return WEST;
            case SOUTH:
                return NORTH;
            case WEST:
                return EAST;
            default:
                return 0;
        }
    }

    protected static double distance(Point ptA, Point ptB)
    {
        return Math.sqrt(Math.pow(ptA.x - ptB.x, 2) + Math.pow(ptA.y - ptB.y, 2));
    }

    // joins two arrays of objects
    protected static Point[] join(Point[] a, Point[] b)
    {
        Point[] result = new Point[a.length + b.length];
        System.arraycopy(a, 0, result, 0, a.length);
        System.arraycopy(b, 0, result, a.length, b.length);
        return result;
    }
}
