package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Architecture;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

/**
 * This property specifies whether the related frame is a single
 * frame, or an array of frames. In the later case, the property
 * of a parent frame of architecture representing the size of the
 * array is to be chosen.
 *
 * @author Michael Cifka
 */
public class PropFrameArray extends PropertySupport.ReadWrite
{

    private static final String NOT_ARRAY = "(not an array)";
    private Component component;

    public PropFrameArray(Component component)
    {
        super("array", String.class, "Array", null);
        this.component = component;
    }


    public Object getValue()
    {
        return component.getArrayProperty();
    }

    public void setValue(Object val)
    {
        if(component.getParent() == null) return;
        if(component.getParent().getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited architecture.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        component.setArrayProperty((String) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return new ArrayPropertyEditor();
    }

    private class ArrayPropertyEditor extends PropertyEditorSupport
    {
        public String[] getTags()
        {
            java.util.ArrayList tags = new java.util.ArrayList();
            tags.add(NOT_ARRAY);
            if(component.getParent() == null)
                return (String[]) tags.toArray(new String[0]);

            Architecture architecture = component.getParent();
            Component parentComp = architecture.getParent();

            // properties defined by the super architecture
            String[] archProps = architecture.getProperties();
            for(int i = 0; i < archProps.length; i++)
                if(architecture.getPropertyType(archProps[i]).equals("integer"))
                    tags.add(archProps[i]);

            // properties defined by the super frame
            String[] parentProps = parentComp.getProperties();
            for(int i = 0; i < parentProps.length; i++)
                if(parentComp.getPropertyType(parentProps[i]).equals("integer"))
                    tags.add(parentProps[i]);

            return (String[]) tags.toArray(new String[0]);
        }

        public String getAsText()
        {
            String value = (String) PropFrameArray.this.getValue();
            return (value == null) ? NOT_ARRAY : value;
        }

        public void setAsText(String s)
        {
            PropFrameArray.this.setValue(s.equals(NOT_ARRAY) ? null : s);
        }
    }
}