package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyEditor;

/**
 * Using this property, the developer can provide a custom source code,
 * the protocol of the related frame. The code is entered via the provided
 * text editting window and it will be included in the generated CDL file,
 * under the protocol clause of the related frame.
 *
 * @author Michael Cifka
 */
public class PropFrameProtocol extends PropertySupport.ReadWrite
{
    private Component comp;

    public PropFrameProtocol(Component comp)
    {
        super("protocol", String.class, "Protocol", null);
        setValue("canEditAsText", Boolean.FALSE);
        setValue("changeImmediate", Boolean.FALSE);
        this.comp = comp;
    }

    public Object getValue()
    {
        return comp.getType().getProtocolCode();
    }

    public void setValue(Object val)
    {
        if(comp.getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited frame.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        comp.getType().setProtocolCode((String) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return new cz.cuni.dsrg.sofa.netbeans.visualdesigner.text.SourceCodeEditor();
    }
}