package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.typemanager.TypeEditor;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.FrameType;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;

/**
 * A property representing the frame type that was choosen
 * as the type of the component.
 *
 * @author Michael Cifka
 */
public class PropFrameType extends PropertySupport.ReadWrite
{
    private Component comp;
    private TypeEditor editor;

    public PropFrameType(Component c)
    {
        super("frame", FrameType.class, "Type", "Type of the frame implemented by this component.");
        setValue("changeImmediate", Boolean.FALSE);
        comp = c;
        editor = new TypeEditor(comp.getRepository());
        editor.addPropertyChangeListener(new PropertyChangeListener()
        {
            public void propertyChange(PropertyChangeEvent evt)
            {
                if(evt.getPropertyName() != null
                        && evt.getPropertyName().equals("setValue"))
                    setValue(editor.getValue());
            }
        });
        editor.setRequiredClass(FrameType.class);
    }

    public Object getValue()
    {
        return comp.getType();
    }

    public void setValue(Object val)
    {
        if(comp.getParent() != null &&
                comp.getParent().getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited architecture.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        comp.setType((FrameType) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return editor;
    }
}