package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Interface;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;

/**
 * This property specifies whether the related interface is a single
 * interface, or an array of interfaces. In the later case, the property
 * of a parent frame representing the size of the array is to be chosen.
 *
 * @author Michael Cifka
 */
public class PropInterfaceArray extends PropertySupport.ReadWrite
{

    private static final String NOT_ARRAY = "(not an array)";
    private Interface iface;

    public PropInterfaceArray(Interface iface)
    {
        super("array", String.class, "Array", null);
        this.iface = iface;
    }


    public Object getValue()
    {
        return iface.getArrayProperty();
    }

    public void setValue(Object val)
    {
        if(iface.getParent().getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited frame.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        iface.setArrayProperty((String) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return new ArrayPropertyEditor();
    }

    private class ArrayPropertyEditor extends PropertyEditorSupport
    {
        public String[] getTags()
        {
            java.util.ArrayList tags = new java.util.ArrayList();
            tags.add(NOT_ARRAY);
            if(iface.getParent() == null)
                return (String[]) tags.toArray(new String[0]);

            Component parentComp = iface.getParent();

            // properties defined by the frame
            String[] parentProps = parentComp.getProperties();
            for(int i = 0; i < parentProps.length; i++)
                if(parentComp.getPropertyType(parentProps[i]).equals("integer"))
                    tags.add(parentProps[i]);

            return (String[]) tags.toArray(new String[0]);
        }

        public String getAsText()
        {
            String value = (String) PropInterfaceArray.this.getValue();
            return (value == null) ? NOT_ARRAY : value;
        }

        public void setAsText(String s)
        {
            PropInterfaceArray.this.setValue(s.equals(NOT_ARRAY) ? null : s);
        }
    }
}