package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Interface;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyEditor;

/**
 * Using this property, the developer can provide a custom source code
 * for the body of an interface. The code is entered via the provided
 * text editting window and it will be inserted inside the interface
 * definition.
 *
 * @author Michael Cifka
 */

public class PropInterfaceSource extends PropertySupport.ReadWrite
{
    private Interface iface;

    public PropInterfaceSource(Interface iface)
    {
        super("source", String.class, "Source Code", null);
        setValue("canEditAsText", Boolean.FALSE);
        setValue("changeImmediate", Boolean.FALSE);
        this.iface = iface;
    }

    public Object getValue()
    {
        return iface.getType().getSourceCode();
    }

    public void setValue(Object val)
    {
        if(iface.getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited interface.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        iface.getType().setSourceCode((String) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return new cz.cuni.dsrg.sofa.netbeans.visualdesigner.text.SourceCodeEditor();
    }
}