package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import org.openide.nodes.PropertySupport;

/**
 * A property representing the location, reference to a server
 * where the component should be run. Set of subcomponents with
 * the value of this property makes up a deployment unit.
 *
 * @author Michael Cifka
 */

public class PropLocation extends PropertySupport.ReadWrite
{

    private Component component;
    private static final String NOT_SPECIFIED = "(not specified)";
    private static final String SAME_AS_PARENT = "(same as parent)";


    public PropLocation(Component component)
    {
        super("location", String.class, "Location", null);
        setValue("canEditAsText", Boolean.TRUE);
        this.component = component;
    }


    public Object getValue()
    {
        String s = component.getLocation();
        if(s == null)
            return (component.getParent() == null)
                    ? NOT_SPECIFIED : SAME_AS_PARENT;
        else
            return s;
    }

    public void setValue(Object val)
    {
        String s = (String) val;
        if(component.getParent() != null
                && s.equals(component.getParent().getParent().getLocation()))
            component.setLocation(null);
        else if(s.equals("") || s.equals(NOT_SPECIFIED) || s.equals(SAME_AS_PARENT))
            component.setLocation(null);
        else
            component.setLocation(s);
    }

}