package cz.cuni.dsrg.sofa.netbeans.visualdesigner.loaders;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.nodes.ComponentNode;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.Repository;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.Node;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * The data object for the SOFA Component Project (.sofa) file.
 * It allows the file to be displayed as a component node, containing
 * the tree-hierarchy of subcomponent node, with a set of various
 * operations available.
 *
 * @author Michael Cifka
 */
public class ComponentDataObject extends MultiDataObject implements SaveCookie
{

    Component component;
    Repository repository;

    public ComponentDataObject(FileObject primaryFile, ComponentDataLoader loader) throws DataObjectExistsException, IOException
    {
        super(primaryFile, loader);
///        MiscHelps.message("Konec konstruktoru, volal super: ComponentDataObject");
///        MiscHelps.message("Primary file: " + primaryFile.toString());
        component = load(primaryFile);
       if(component == null)
        {
///          MiscHelps.message("Primary file (repository) je null");
            repository = new Repository();
            component = new Component(primaryFile.getName(), repository);
        }
        else
        {
            repository = component.getRepository();
        }
    }

    protected Node createNodeDelegate()
    {
        if(isTemplate())
            return super.createNodeDelegate();
        else
            return new ComponentNode(component, this);
    }

    private Component load(FileObject fo)
    {
        Component c;
        try
        {
            ObjectInputStream in = new ObjectInputStream(fo.getInputStream());
///   if(in==null)MiscHelps.message("IN je null"); else MiscHelps.message("IN neni null");
            c = (Component) in.readObject();
///  if(c==null)MiscHelps.message("Component je null"); else MiscHelps.message("Component neni null");
            in.close();
        }
        catch (Exception ex)
        {
            c = null;
        }
        return c;
    }

    public void save() throws IOException
    {
        setModified(false);
        FileObject fo = getPrimaryFile();
        FileLock lock = fo.lock();
        ObjectOutputStream out = new ObjectOutputStream(fo.getOutputStream(lock));
        out.writeObject(component);
        out.flush();
        out.close();
        lock.releaseLock();
    }

    public void rename2(String s) throws IOException
    {
        FileObject foComp = getPrimaryFile();
        FileObject foCDL = FileUtil.findBrother(foComp, "cdl");
        FileObject foDD = FileUtil.findBrother(foComp, "dd");
        super.rename(s);

        // also try to rename the generated files, if existing
        if(foCDL != null)
        {
            try
            {
                FileLock lock = foCDL.lock();
                foCDL.rename(lock, s, "cdl");
                lock.releaseLock();
            }
            catch (IOException ex)
            {
            }
            ;
        }
        if(foDD != null)
        {
            try
            {
                FileLock lock = foDD.lock();
                foDD.rename(lock, s, "dd");
                lock.releaseLock();
            }
            catch (IOException ex)
            {
            }
            ;
        }
    }

    public void delete2() throws IOException
    {
        FileObject foComp = getPrimaryFile();
        FileObject foCDL = FileUtil.findBrother(foComp, "cdl");
        FileObject foDD = FileUtil.findBrother(foComp, "dd");
        super.delete();
        setModified(false);

        // also try to delete the generated files, if existing
        if(foCDL != null)
            try
            {
                foCDL.delete();
            }
            catch (IOException ex)
            {
            }
        ;
        if(foDD != null)
        {
            try
            {
                foDD.delete();
            }
            catch (IOException ex)
            {
            }
            ;
        }
    }
}