package cz.cuni.dsrg.sofa.netbeans.visualdesigner.text;

import org.openide.explorer.propertysheet.ExPropertyEditor;
import org.openide.explorer.propertysheet.PropertyEnv;

import javax.swing.*;
import java.awt.*;
import java.awt.event.FocusAdapter;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * This class implements an editing window used to edit properties
 * that are multiple lines of a custom source code.
 *
 * @author Michael Cifka
 */
public class SourceCodeEditor implements ExPropertyEditor, PropertyChangeListener
{

    private String text = null;
    private JEditorPane editor;
    private JScrollPane scroll;


    private PropertyChangeSupport supp = new PropertyChangeSupport(this);

    public SourceCodeEditor()
    {
        editor = new JEditorPane();
        editor.setFont(new java.awt.Font("DialogInput", 0, 12));
        editor.addFocusListener(new FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                propertyChange(null);
            }
        });
        editor.addPropertyChangeListener(this);
        scroll = new JScrollPane();
        scroll.setViewportView(editor);
        scroll.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(12, 12, 0, 12), scroll.getBorder()));
        scroll.setPreferredSize(new Dimension(500, 310));
    }

    public void addPropertyChangeListener(PropertyChangeListener l)
    {
        supp.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l)
    {
        supp.removePropertyChangeListener(l);
    }

    public boolean supportsCustomEditor()
    {
        return true;
    }

    public Component getCustomEditor()
    {
        return scroll;
    }

    public void propertyChange(PropertyChangeEvent ev)
    {
        supp.firePropertyChange(null, null, null);
    }

    public Object getValue()
    {
        return editor.getText();
    }

    public void setValue(Object obj)
    {
        if(obj == null)
            editor.setText("");
        else
            editor.setText((String) obj);
    }

    public String getAsText()
    {
        String text = editor.getText();
        if(text.equals(""))
            return "(empty)";
        int index = text.indexOf("\n"), lines = 1;
        while(index != -1)
        {
            lines++;
            index = text.indexOf("\n", index + 1);
        }
        if(lines == 1)
            return "1 line";
        else
            return lines + " lines";
    }

    public void setAsText(String text)
    {
    }


    public String getJavaInitializationString()
    {
        return null;
    }

    public String[] getTags()
    {
        return null;
    }

    public boolean isPaintable()
    {
        return false;
    }

    public void paintValue(Graphics g, Rectangle r)
    {
    }

    public void attachEnv(PropertyEnv propertyEnv)
    {
    }
}