package cz.cuni.dsrg.sofa.netbeans.visualdesigner.text;

import org.openide.actions.OpenAction;
import org.openide.filesystems.FileObject;
import org.openide.loaders.*;
import org.openide.nodes.Children;
import org.openide.nodes.CookieSet;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;

/**
 * This class, duplicating the original NetBeans source code, allows
 * to edit a source file in the standard way.
 *
 * @author Michael Cifka
 */
public class TXTDataObject extends MultiDataObject implements CookieSet.Factory
{

    private transient TXTEditorSupport editorSupport;


    public TXTDataObject(final FileObject obj, final MultiFileLoader loader) throws DataObjectExistsException
    {
        super(obj, loader);

        getCookieSet().add(TXTEditorSupport.class, this);
    }


    public Node.Cookie createCookie(Class clazz)
    {
        if(clazz.isAssignableFrom(TXTEditorSupport.class))
            return getEditorSupport();
        else
            return null;
    }

    CookieSet getCookieSet0()
    {
        return getCookieSet();
    }

    private TXTEditorSupport getEditorSupport()
    {
        if(editorSupport == null)
        {
            synchronized (this)
            {
                if(editorSupport == null)
                    editorSupport = new TXTEditorSupport(this);
            }
        }

        return editorSupport;
    }

    protected Node createNodeDelegate()
    {
        return new TXTNode(this, iconBase());
    }

    public HelpCtx getHelpCtx()
    {
        return new HelpCtx(TXTDataObject.class);
    }

    protected String iconBase()
    {
        return "org/netbeans/modules/text/txtObject";
    }


    public static final class TXTNode extends DataNode
    {
        public TXTNode(final DataObject dataObject, String iconBase)
        {
            super(dataObject, Children.LEAF);
            setIconBase(iconBase);
        }

        public SystemAction getDefaultAction()
        {
            SystemAction result = super.getDefaultAction();
            return result == null ? SystemAction.get(OpenAction.class) : result;
        }
    }

}
