package cz.cuni.dsrg.sofa.netbeans.visualdesigner.text;

import org.openide.cookies.*;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.nodes.Node.Cookie;
import org.openide.text.DataEditorSupport;
import org.openide.windows.CloneableOpenSupport;

import java.io.IOException;

/**
 * This class, duplicating the original NetBeans source code, allows
 * to edit a source file in the standard way.
 *
 * @author Michael Cifka
 */
public final class TXTEditorSupport extends DataEditorSupport
        implements OpenCookie, EditCookie, EditorCookie, PrintCookie, CloseCookie
{

    private final SaveCookie saveCookie = new SaveCookie()
    {
        public void save() throws IOException
        {
            TXTEditorSupport.this.saveDocument();
            TXTEditorSupport.this.getDataObject().setModified(false);
        }
    };


    TXTEditorSupport(TXTDataObject obj)
    {
        super(obj, new Environment(obj));

        setMIMEType("text/plain");
    }

    protected boolean notifyModified()
    {
        if(!super.notifyModified())
            return false;

        addSaveCookie();

        return true;
    }

    protected void notifyUnmodified()
    {
        super.notifyUnmodified();

        removeSaveCookie();
    }

    private void addSaveCookie()
    {
        TXTDataObject obj = (TXTDataObject) getDataObject();

        if(obj.getCookie(SaveCookie.class) == null)
        {
            obj.getCookieSet0().add(saveCookie);
            obj.setModified(true);
        }
    }

    private void removeSaveCookie()
    {
        TXTDataObject obj = (TXTDataObject) getDataObject();

        Cookie cookie = obj.getCookie(SaveCookie.class);

        if(cookie != null && cookie.equals(saveCookie))
        {
            obj.getCookieSet0().remove(saveCookie);
            obj.setModified(false);
        }
    }


    private static class Environment extends DataEditorSupport.Env
    {

        public Environment(TXTDataObject obj)
        {
            super(obj);
        }


        protected FileObject getFile()
        {
            return getDataObject().getPrimaryFile();
        }

        protected FileLock takeLock() throws IOException
        {
            return ((TXTDataObject) getDataObject()).getPrimaryEntry().takeLock();
        }

        public CloneableOpenSupport findCloneableOpenSupport()
        {
            return (TXTEditorSupport) getDataObject().getCookie(TXTEditorSupport.class);
        }
    }

}
