package cz.cuni.dsrg.sofa.netbeans.visualdesigner.types;

/**
 * This class extends the {@link AbstractType} to hold, import, and export
 * the type information representing a module, a folder able to hold and
 * organize other type elements.<p>
 *
 * This class can be persisted using the Java Object Serialization.
 *
 * @author Michael Cifka
 */
public class Module extends AbstractType implements java.io.Serializable
{

    Module()
    {
    }

    public Module(String name)
    {
        this.name = name;
    }

    public String getFullName()
    {
        if(parent instanceof Repository)
            return "::" + getName();
        else
            return parent.getFullName() + "::" + getName();
    }

    String writeInterfaceTypesAsCDL(int indent)
    {
        String indentString = indentString(indent);
        String s = "";

        AbstractType[] ch = getChildren();
        // write interface types
        for(int i = 0; i < ch.length; i++)
            if(ch[i] instanceof InterfaceType)
                s += ch[i].writeAsCDL(indent + 1);
        // write modules
        for(int i = 0; i < ch.length; i++)
            if(ch[i] instanceof Module)
                s += ((Module) ch[i]).writeInterfaceTypesAsCDL(indent + 1);

        String t = (parent instanceof Repository) ? "" : "\n";
        if(s.equals(""))
            return "";
        else
            return t + indentString + "module " + getName() + " {" + s
                    + indentString + "};\n";
    }

    String writeFrameTypesAsCDL(int indent)
    {
        String indentString = indentString(indent);
        String s = "";

        AbstractType[] ch = getChildren();
        // write interface types
        for(int i = 0; i < ch.length; i++)
            if(ch[i] instanceof FrameType)
                s += ch[i].writeAsCDL(indent + 1);
        // write modules
        for(int i = 0; i < ch.length; i++)
            if(ch[i] instanceof Module)
                s += ((Module) ch[i]).writeFrameTypesAsCDL(indent + 1);


        String t = (parent instanceof Repository) ? "" : "\n";
        if(s.equals(""))
            return "";
        else
            return t + indentString + "module " + getName() + " {" + s
                    + indentString + "};\n";

    }
}